@@ -14,22 +14,25 @@
 // You should have received a copy of the GNU General Public License
 // along with this program. If not, see <http://www.gnu.org/licenses/>.
 
-use std::cmp::{max, Reverse};
-use std::fs;
-use std::io;
-use std::path::{Path, PathBuf};
-use std::str::FromStr;
+use std::{
+    cmp::{max, Reverse},
+    fs, io,
+    path::{Path, PathBuf},
+    str::FromStr,
+};
 
 use arrayvec::ArrayVec;
 use once_cell::sync::OnceCell;
-use rustc_hash::FxHashMap;
 use positioned_io::RandomAccessFile;
-
+use rustc_hash::FxHashMap;
 use shakmaty::{Material, Move, Position, Role};
 
-use crate::errors::{ProbeResultExt as _, SyzygyError, SyzygyResult};
-use crate::table::{DtzTable, WdlTable};
-use crate::types::{DecisiveWdl, Dtz, Metric, Syzygy, Wdl};
+use crate::{
+    errors::{ProbeResultExt as _, SyzygyError, SyzygyResult},
+    table::{DtzTable, WdlTable},
+    types::{DecisiveWdl, Dtz, MaybeRounded, Metric, Syzygy, Wdl},
+    AmbiguousWdl,
+};
 
 /// Additional probe information from a brief alpha-beta search.
 #[derive(Debug, Copy, Clone, Eq, PartialEq)]
@@ -48,6 +51,7 @@ enum ProbeState {
 pub struct Tablebase<S: Position + Clone + Syzygy> {
     wdl: FxHashMap<Material, (PathBuf, OnceCell<WdlTable<S, RandomAccessFile>>)>,
     dtz: FxHashMap<Material, (PathBuf, OnceCell<DtzTable<S, RandomAccessFile>>)>,
+    max_pieces: usize,
 }
 
 impl<S: Position + Clone + Syzygy> Default for Tablebase<S> {
@@ -62,9 +66,15 @@ impl<S: Position + Clone + Syzygy> Tablebase<S> {
         Tablebase {
             wdl: FxHashMap::with_capacity_and_hasher(145, Default::default()),
             dtz: FxHashMap::with_capacity_and_hasher(145, Default::default()),
+            max_pieces: 0,
         }
     }
 
+    /// Returns the maximum number of pieces of any added table.
+    pub fn max_pieces(&self) -> usize {
+        self.max_pieces
+    }
+
     /// Add all relevant tables from a directory.
     ///
     /// Tables are selected by filename, e.g. `KQvKP.rtbz`. The files are not
@@ -122,7 +132,9 @@ impl<S: Position + Clone + Syzygy> Tablebase<S> {
             _ => return Err(io::Error::from(io::ErrorKind::InvalidInput)),
         };
 
-        if material.count() > S::MAX_PIECES {
+        let pieces = material.count();
+
+        if pieces > S::MAX_PIECES {
             return Err(io::Error::from(io::ErrorKind::InvalidInput));
         }
 
@@ -130,20 +142,33 @@ impl<S: Position + Clone + Syzygy> Tablebase<S> {
             return Err(io::Error::from(io::ErrorKind::InvalidInput));
         }
 
-        if ext == S::TBW.ext || (!material.has_pawns() && S::PAWNLESS_TBW.map_or(false, |t| ext == t.ext)) {
-            self.wdl.insert(material, (path.to_path_buf(), OnceCell::new()));
-        } else if ext == S::TBZ.ext || (!material.has_pawns() && S::PAWNLESS_TBZ.map_or(false, |t| ext == t.ext)) {
-            self.dtz.insert(material, (path.to_path_buf(), OnceCell::new()));
+        if ext == S::TBW.ext
+            || (!material.has_pawns() && S::PAWNLESS_TBW.map_or(false, |t| ext == t.ext))
+        {
+            self.wdl
+                .insert(material, (path.to_path_buf(), OnceCell::new()));
+        } else if ext == S::TBZ.ext
+            || (!material.has_pawns() && S::PAWNLESS_TBZ.map_or(false, |t| ext == t.ext))
+        {
+            self.dtz
+                .insert(material, (path.to_path_buf(), OnceCell::new()));
         } else {
             return Err(io::Error::from(io::ErrorKind::InvalidInput));
         }
 
+        self.max_pieces = max(self.max_pieces, pieces);
         Ok(())
     }
 
     fn wdl_table(&self, key: &Material) -> SyzygyResult<&WdlTable<S, RandomAccessFile>> {
-        if let Some(&(ref path, ref table)) = self.wdl.get(key).or_else(|| self.wdl.get(&key.clone().into_flipped())) {
-            table.get_or_try_init(|| WdlTable::open(path, key)).ctx(Metric::Wdl, key)
+        if let Some(&(ref path, ref table)) = self
+            .wdl
+            .get(key)
+            .or_else(|| self.wdl.get(&key.clone().into_flipped()))
+        {
+            table
+                .get_or_try_init(|| WdlTable::open(path, key))
+                .ctx(Metric::Wdl, key)
         } else {
             Err(SyzygyError::MissingTable {
                 metric: Metric::Wdl,
@@ -153,8 +178,14 @@ impl<S: Position + Clone + Syzygy> Tablebase<S> {
     }
 
     fn dtz_table(&self, key: &Material) -> SyzygyResult<&DtzTable<S, RandomAccessFile>> {
-        if let Some(&(ref path, ref table)) = self.dtz.get(key).or_else(|| self.dtz.get(&key.clone().into_flipped())) {
-            table.get_or_try_init(|| DtzTable::open(path, key)).ctx(Metric::Dtz, key)
+        if let Some(&(ref path, ref table)) = self
+            .dtz
+            .get(key)
+            .or_else(|| self.dtz.get(&key.clone().into_flipped()))
+        {
+            table
+                .get_or_try_init(|| DtzTable::open(path, key))
+                .ctx(Metric::Dtz, key)
         } else {
             Err(SyzygyError::MissingTable {
                 metric: Metric::Dtz,
@@ -163,44 +194,53 @@ impl<S: Position + Clone + Syzygy> Tablebase<S> {
         }
     }
 
-    /// Probe tables for the [`Wdl`] value of a position.
-    ///
-    /// This indicates if the position is winning, lost or drawn with
-    /// or without the 50-move rule, assuming `pos` is reached directly after
-    /// a capture or pawn move.
+    /// Probe tables for the [`Wdl`] value of a position, assuming `pos`
+    /// is reached directly after a capture or pawn move.
     ///
     /// # Errors
     ///
-    /// See [`SyzygyError`] for possible error
-    /// conditions.
-    pub fn probe_wdl(&self, pos: &S) -> SyzygyResult<Wdl> {
-        self.probe(pos).map(|entry| entry.wdl())
+    /// See [`SyzygyError`] for possible error conditions.
+    pub fn probe_wdl_after_zeroing(&self, pos: &S) -> SyzygyResult<Wdl> {
+        self.probe(pos).map(|entry| entry.wdl_after_zeroing())
+    }
+
+    /// Probe tables for the WDL value of a position, considering also
+    /// the halfmove counter of `pos`. The result may be
+    /// [ambiguous due to DTZ rounding](MaybeRounded).
+    ///
+    /// Requires both WDL and DTZ tables.
+    ///
+    /// # Errors
+    ///
+    /// See [`SyzygyError`] for possible error conditions.
+    pub fn probe_wdl(&self, pos: &S) -> SyzygyResult<AmbiguousWdl> {
+        self.probe(pos)
+            .and_then(|entry| entry.dtz())
+            .map(|dtz| AmbiguousWdl::from_dtz_and_halfmoves(dtz, pos.halfmoves()))
     }
 
     /// Probe tables for the [`Dtz`] value of a position.
     ///
-    /// Min-maxing the DTZ of the available moves guarantees achieving the
-    /// optimal outcome under the 50-move rule.
-    ///
     /// Requires both WDL and DTZ tables.
     ///
     /// # Errors
     ///
-    /// See [`SyzygyError`] for possible error
-    /// conditions.
-    pub fn probe_dtz(&self, pos: &S) -> SyzygyResult<Dtz> {
+    /// See [`SyzygyError`] for possible error conditions.
+    pub fn probe_dtz(&self, pos: &S) -> SyzygyResult<MaybeRounded<Dtz>> {
         self.probe(pos).and_then(|entry| entry.dtz())
     }
 
-    /// Select a DTZ-optimal move.
+    /// Select a [`Dtz`]-optimal move.
+    ///
+    /// Following the mainline starting from a capture or pawn move guarantees
+    /// achieving the optimal outcome under the 50-move rule.
     ///
     /// Requires both WDL and DTZ tables.
     ///
     /// # Errors
     ///
-    /// See [`SyzygyError`] for possible error
-    /// conditions.
-    pub fn best_move(&self, pos: &S) -> SyzygyResult<Option<(Move, Dtz)>> {
+    /// See [`SyzygyError`] for possible error conditions.
+    pub fn best_move(&self, pos: &S) -> SyzygyResult<Option<(Move, MaybeRounded<Dtz>)>> {
         struct WithAfter<S> {
             m: Move,
             after: S,
@@ -215,39 +255,65 @@ impl<S: Position + Clone + Syzygy> Tablebase<S> {
             m: Move,
             immediate_loss: bool,
             zeroing: bool,
-            dtz: Dtz,
+            dtz: MaybeRounded<Dtz>,
         }
 
         // Build list of successor positions.
-        let with_after = pos.legal_moves().into_iter().map(|m| {
-            let mut after = pos.clone();
-            after.play_unchecked(&m);
-            WithAfter { m, after }
-        }).collect::<ArrayVec<_, 256>>();
+        let with_after = pos
+            .legal_moves()
+            .into_iter()
+            .map(|m| {
+                let mut after = pos.clone();
+                after.play_unchecked(&m);
+                WithAfter { m, after }
+            })
+            .collect::<ArrayVec<_, 256>>();
 
         // Determine WDL for each move.
-        let with_wdl = with_after.iter().map(|e| Ok(WithWdlEntry {
-            m: e.m.clone(),
-            entry: self.probe(&e.after)?,
-        })).collect::<SyzygyResult<ArrayVec<_, 256>>>()?;
+        let with_wdl = with_after
+            .iter()
+            .map(|e| {
+                Ok(WithWdlEntry {
+                    m: e.m.clone(),
+                    entry: self.probe(&e.after)?,
+                })
+            })
+            .collect::<SyzygyResult<ArrayVec<_, 256>>>()?;
 
         // Find best WDL.
-        let best_wdl = with_wdl.iter().map(|a| a.entry.wdl).min().unwrap_or(Wdl::Loss);
+        let best_wdl = with_wdl
+            .iter()
+            .map(|a| a.entry.wdl)
+            .min()
+            .unwrap_or(Wdl::Loss);
 
         // Select a DTZ-optimal move among the moves with best WDL.
-        itertools::process_results(with_wdl.iter().filter(|a| a.entry.wdl == best_wdl).map(|a| {
-            let dtz = a.entry.dtz()?;
-            Ok(WithDtz {
-                immediate_loss: dtz == Dtz(-1) && (a.entry.pos.is_checkmate() || a.entry.pos.variant_outcome().is_some()),
-                zeroing: a.m.is_zeroing(),
-                m: a.m.clone(),
-                dtz,
-            })
-        }), |iter| iter.min_by_key(|m| (
-            Reverse(m.immediate_loss),
-            m.zeroing ^ (m.dtz < Dtz(0)), // zeroing is good/bad if winning/losing
-            Reverse(m.dtz),
-        )).map(|m| (m.m, m.dtz)))
+        itertools::process_results(
+            with_wdl
+                .iter()
+                .filter(|a| a.entry.wdl == best_wdl)
+                .map(|a| {
+                    let dtz = a.entry.dtz()?;
+                    Ok(WithDtz {
+                        immediate_loss: dtz.ignore_rounding() == Dtz(-1)
+                            && (a.entry.pos.is_checkmate()
+                                || a.entry.pos.variant_outcome().is_some()),
+                        zeroing: a.m.is_zeroing(),
+                        m: a.m.clone(),
+                        dtz,
+                    })
+                }),
+            |iter| {
+                iter.min_by_key(|m| {
+                    (
+                        Reverse(m.immediate_loss),
+                        m.zeroing ^ (m.dtz.ignore_rounding() < Dtz(0)), // zeroing is good/bad if winning/losing
+                        Reverse(m.dtz.ignore_rounding()),
+                    )
+                })
+                .map(|m| (m.m, m.dtz))
+            },
+        )
     }
 
     fn probe<'a>(&'a self, pos: &'a S) -> SyzygyResult<WdlEntry<'a, S>> {
@@ -352,8 +418,12 @@ impl<S: Position + Clone + Syzygy> Tablebase<S> {
                 tablebase: self,
                 pos,
                 wdl: best_capture,
-                state: if best_capture > Wdl::Draw { ProbeState::ZeroingBestMove } else { ProbeState::Normal },
-            })
+                state: if best_capture > Wdl::Draw {
+                    ProbeState::ZeroingBestMove
+                } else {
+                    ProbeState::Normal
+                },
+            });
         }
 
         // If the position would be stalemate without ep captures, then we are
@@ -364,7 +434,7 @@ impl<S: Position + Clone + Syzygy> Tablebase<S> {
                 pos,
                 wdl: best_ep,
                 state: ProbeState::ZeroingBestMove,
-            })
+            });
         }
 
         Ok(WdlEntry {
@@ -394,11 +464,20 @@ impl<S: Position + Clone + Syzygy> Tablebase<S> {
         Ok(max(alpha, v))
     }
 
-    fn probe_compulsory_captures(&self, pos: &S, mut alpha: Wdl, beta: Wdl, threats: bool) -> SyzygyResult<(Wdl, ProbeState)> {
+    fn probe_compulsory_captures(
+        &self,
+        pos: &S,
+        mut alpha: Wdl,
+        beta: Wdl,
+        threats: bool,
+    ) -> SyzygyResult<(Wdl, ProbeState)> {
         assert!(S::CAPTURES_COMPULSORY);
 
         if let Some(outcome) = pos.variant_outcome() {
-            return Ok((Wdl::from_outcome(outcome, pos.turn()), ProbeState::ZeroingBestMove));
+            return Ok((
+                Wdl::from_outcome(outcome, pos.turn()),
+                ProbeState::ZeroingBestMove,
+            ));
         }
 
         // Explore compulsory captures in antichess variants.
@@ -442,7 +521,14 @@ impl<S: Position + Clone + Syzygy> Tablebase<S> {
         if v > alpha {
             Ok((v, ProbeState::Normal))
         } else {
-            Ok((alpha, if threats_found { ProbeState::Threat } else { ProbeState::Normal }))
+            Ok((
+                alpha,
+                if threats_found {
+                    ProbeState::Threat
+                } else {
+                    ProbeState::Normal
+                },
+            ))
         }
     }
 
@@ -466,7 +552,11 @@ impl<S: Position + Clone + Syzygy> Tablebase<S> {
             }
         }
 
-        Ok(if captures.is_empty() { None } else { Some(alpha) })
+        Ok(if captures.is_empty() {
+            None
+        } else {
+            Some(alpha)
+        })
     }
 
     fn probe_wdl_table(&self, pos: &S) -> SyzygyResult<Wdl> {
@@ -486,7 +576,11 @@ impl<S: Position + Clone + Syzygy> Tablebase<S> {
             .and_then(|table| table.probe_wdl(pos).ctx(Metric::Wdl, &key))
     }
 
-    fn probe_dtz_table(&self, pos: &S, wdl: DecisiveWdl) -> SyzygyResult<Option<Dtz>> {
+    fn probe_dtz_table(
+        &self,
+        pos: &S,
+        wdl: DecisiveWdl,
+    ) -> SyzygyResult<Option<MaybeRounded<u32>>> {
         // Get raw DTZ value from the appropriate table.
         let key = pos.board().material();
         self.dtz_table(&key)
@@ -504,23 +598,25 @@ struct WdlEntry<'a, S: Position + Clone + Syzygy> {
 }
 
 impl<'a, S: Position + Clone + Syzygy + 'a> WdlEntry<'a, S> {
-    fn wdl(&self) -> Wdl {
+    fn wdl_after_zeroing(&self) -> Wdl {
         self.wdl
     }
 
-    fn dtz(&self) -> SyzygyResult<Dtz> {
+    fn dtz(&self) -> SyzygyResult<MaybeRounded<Dtz>> {
         let wdl = match self.wdl.decisive() {
             Some(wdl) => wdl,
-            None => return Ok(Dtz(0)),
+            None => return Ok(MaybeRounded::Precise(Dtz(0))),
         };
 
         if self.state == ProbeState::ZeroingBestMove || self.pos.us() == self.pos.our(Role::Pawn) {
-            return Ok(Dtz::before_zeroing(wdl));
+            return Ok(MaybeRounded::Precise(Dtz::before_zeroing(wdl.into())));
         }
 
         if self.state == ProbeState::Threat && wdl >= DecisiveWdl::CursedWin {
             // The position is a win or a cursed win by a threat move.
-            return Ok(Dtz::before_zeroing(wdl).add_plies(1));
+            return Ok(MaybeRounded::Precise(
+                Dtz::before_zeroing(wdl.into()).add_plies(1),
+            ));
         }
 
         // If winning, check for a winning pawn move. No need to look at
@@ -532,41 +628,43 @@ impl<'a, S: Position + Clone + Syzygy + 'a> WdlEntry<'a, S> {
             for m in &pawn_advances {
                 let mut after = self.pos.clone();
                 after.play_unchecked(m);
-                let v = -self.tablebase.probe_wdl(&after)?;
+                let v = -self.tablebase.probe_wdl_after_zeroing(&after)?;
                 if v == wdl.into() {
-                    return Ok(Dtz::before_zeroing(wdl));
+                    return Ok(MaybeRounded::Precise(Dtz::before_zeroing(wdl.into())));
                 }
             }
         }
 
         // At this point we know that the best move is not a capture. Probe the
         // table. DTZ tables store only one side to move.
-        if let Some(Dtz(dtz)) = self.tablebase.probe_dtz_table(self.pos, wdl)? {
-            return Ok(Dtz::before_zeroing(wdl).add_plies(dtz));
+        if let Some(plies) = self.tablebase.probe_dtz_table(self.pos, wdl)? {
+            return Ok(plies.map(|plies| Dtz::before_zeroing(wdl.into()).add_plies(plies)));
         }
 
-        // We have to probe the other side by doing a 1-ply search.
+        // We have to probe the other side of the table by doing
+        // a 1-ply search.
         let mut moves = self.pos.legal_moves();
         moves.retain(|m| !m.is_zeroing());
 
         let mut best = if wdl >= DecisiveWdl::CursedWin {
             None
         } else {
-            Some(Dtz::before_zeroing(wdl))
+            Some(MaybeRounded::Precise(Dtz::before_zeroing(wdl.into())))
         };
 
         for m in &moves {
             let mut after = self.pos.clone();
             after.play_unchecked(m);
             let v = -self.tablebase.probe_dtz(&after)?;
-            if v == Dtz(1) && after.is_checkmate() {
-                best = Some(Dtz(1));
-            } else if wdl >= DecisiveWdl::CursedWin {
-                if v > Dtz(0) && best.map_or(true, |best| v + Dtz(1) < best) {
-                    best = Some(v + Dtz(1));
-                }
-            } else if best.map_or(true, |best| v - Dtz(1) < best) {
-                best = Some(v - Dtz(1));
+            if v.ignore_rounding() == Dtz(1) && after.is_checkmate() {
+                best = Some(MaybeRounded::Precise(Dtz(1)));
+            } else if v.signum() == wdl.signum() {
+                let v = v.map(|v| v.add_plies(1));
+                best = match best {
+                    None => Some(v),
+                    Some(best) if v.ignore_rounding() < best.ignore_rounding() => Some(v),
+                    _ => best,
+                };
             }
         }
 
@@ -576,10 +674,9 @@ impl<'a, S: Position + Clone + Syzygy + 'a> WdlEntry<'a, S> {
 
 #[cfg(test)]
 mod tests {
-    use super::*;
+    use shakmaty::{fen::Fen, CastlingMode, Chess, Square};
 
-    use shakmaty::fen::Fen;
-    use shakmaty::{CastlingMode, Chess, Square};
+    use super::*;
 
     #[test]
     fn test_send_sync() {
@@ -593,7 +690,9 @@ mod tests {
     #[test]
     fn test_mating_best_move() {
         let mut tables = Tablebase::new();
-        tables.add_directory("tables/chess").expect("read directory");
+        tables
+            .add_directory("tables/chess")
+            .expect("read directory");
 
         let pos: Chess = "5BrN/8/8/8/8/2k5/8/2K5 b - -"
             .parse::<Fen>()
@@ -601,19 +700,27 @@ mod tests {
             .position(CastlingMode::Chess960)
             .expect("legal position");
 
-        assert!(matches!(tables.best_move(&pos), Ok(Some((Move::Normal {
-            role: Role::Rook,
-            from: Square::G8,
-            capture: None,
-            to: Square::G1,
-            promotion: None,
-        }, Dtz(-1))))));
+        assert!(matches!(
+            tables.best_move(&pos),
+            Ok(Some((
+                Move::Normal {
+                    role: Role::Rook,
+                    from: Square::G8,
+                    capture: None,
+                    to: Square::G1,
+                    promotion: None,
+                },
+                MaybeRounded::Rounded(Dtz(-1))
+            )))
+        ));
     }
 
     #[test]
     fn test_black_escapes_via_underpromotion() {
         let mut tables = Tablebase::new();
-        tables.add_directory("tables/chess").expect("read directory");
+        tables
+            .add_directory("tables/chess")
+            .expect("read directory");
 
         let pos: Chess = "8/6B1/8/8/B7/8/K1pk4/8 b - - 0 1"
             .parse::<Fen>()
@@ -621,20 +728,28 @@ mod tests {
             .position(CastlingMode::Chess960)
             .expect("legal position");
 
-        assert!(matches!(tables.best_move(&pos), Ok(Some((Move::Normal {
-            role: Role::Pawn,
-            from: Square::C2,
-            to: Square::C1,
-            capture: None,
-            promotion: Some(Role::Knight),
-        }, Dtz(109))))));
+        assert!(matches!(
+            tables.best_move(&pos),
+            Ok(Some((
+                Move::Normal {
+                    role: Role::Pawn,
+                    from: Square::C2,
+                    to: Square::C1,
+                    capture: None,
+                    promotion: Some(Role::Knight),
+                },
+                MaybeRounded::Rounded(Dtz(109))
+            )))
+        ));
     }
 
     #[test]
     #[ignore]
     fn test_many_pawns() {
         let mut tables = Tablebase::new();
-        tables.add_directory("tables/chess").expect("read directory");
+        tables
+            .add_directory("tables/chess")
+            .expect("read directory");
 
         let pos: Chess = "3k4/5P2/8/8/4K3/2P3P1/PP6/8 w - - 0 1"
             .parse::<Fen>()
@@ -642,6 +757,9 @@ mod tests {
             .position(CastlingMode::Chess960)
             .expect("legal position");
 
-        assert!(matches!(tables.probe_dtz(&pos), Ok(Dtz(1))));
+        assert!(matches!(
+            tables.probe_dtz(&pos),
+            Ok(MaybeRounded::Precise(Dtz(1)))
+        ));
     }
 }
